<?php

namespace App\Service;

use App\Model\BasicType;
use App\Model\Construction;
use App\Model\Employee;
use App\Model\Product;
use App\Model\ProductActivity;
use App\Model\ProductActivityPrice;
use App\Model\Schedule;
use App\Model\ScheduleDetail;
use App\Model\ScheduleInfo;
use Illuminate\Support\Facades\DB;

class ScheduleService extends Service
{
    public function edit($data,$user){
        list($status,$msg) = $this->scheduleRule($data, $user, false);
        if(!$status) return [$status,$msg];

        try {
            DB::beginTransaction();

            $model = Schedule::where('id',$data['id'])->first();
            $model->start_time = $data['start_time'] ?? 0;
            $model->end_time = $data['end_time'] ?? 0;
            $model->mark = $data['mark'] ?? '';
            $model->save();

            $time = time();
            ScheduleDetail::where('schedule_id',$data['id'])
                ->where('del_time',0)
                ->update(['del_time' => $time]);
            ScheduleInfo::where('schedule_id',$data['id'])
                ->where('del_time',0)
                ->update(['del_time' => $time]);

            if(! empty($data['day_time'])){
                $insert_detail = $insert = [];
                foreach ($data['day_time'] as $value){
                    $insert_detail[] = [
                        'schedule_id' => $model->id,
                        'crt_time' => $time,
                        'start_time' => $value['start_time'] ?? "",
                        'end_time' => $value['end_time'] ?? "",
                        'num' => $value['num'] ?? 0,
                    ];

                    for ($i=0;$i<$value['num'];$i++){
                        foreach ($data['time_range'] as $range){
                            $date = date('Y-m-d',$range);
                            $start_time = strtotime($date . " " .$value['start_time']);
                            $end_time = strtotime($date . " " .$value['end_time']);
                            $insert[] = [
                                'schedule_id' => $model->id,
                                'crt_time' => $time,
                                'day' => $range ?? 0, //0点的时间戳
                                'start_time' => $start_time ?? 0, //加上开始时间段
                                'end_time' => $end_time ?? 0,//加上结束时间段
                            ];
                        }
                    }
                }
                ScheduleDetail::insert($insert_detail);
                ScheduleInfo::insert($insert);
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function add($data,$user){
        list($status,$msg) = $this->scheduleRule($data, $user);
        if(!$status) return [$status,$msg];

        try {
            DB::beginTransaction();

            $model = new Schedule();
            $model->start_time = $data['start_time'] ?? 0;
            $model->end_time = $data['end_time'] ?? 0;
            $model->mark = $data['mark'] ?? '';
            $model->crt_id = $user['id'];
            $model->depart_id = $data['depart_id'] ?? 0;
            $model->top_depart_id = $data['top_depart_id'] ?? 0;
            $model->save();

            $time = time();
            if(! empty($data['day_time'])){
                $insert_detail = $insert = [];
                foreach ($data['day_time'] as $value){
                    $insert_detail[] = [
                        'schedule_id' => $model->id,
                        'crt_time' => $time,
                        'start_time' => $value['start_time'] ?? "",
                        'end_time' => $value['end_time'] ?? "",
                        'num' => $value['num'] ?? 0,
                    ];

                    for ($i=0;$i<$value['num'];$i++){
                        foreach ($data['time_range'] as $range){
                            $date = date('Y-m-d',$range);
                            $start_time = strtotime($date . " " .$value['start_time']);
                            $end_time = strtotime($date . " " .$value['end_time']);
                            $insert[] = [
                                'schedule_id' => $model->id,
                                'crt_time' => $time,
                                'day' => $range ?? 0, //0点的时间戳
                                'start_time' => $start_time ?? 0, //加上开始时间段
                                'end_time' => $end_time ?? 0,//加上结束时间段
                            ];
                        }
                    }
                }
                ScheduleDetail::insert($insert_detail);
                ScheduleInfo::insert($insert);
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function del($data){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];

        $boolean = Construction::where('del_time',0)
            ->where('schedule_id',$data['id'])
            ->exists();
        if($boolean) return [false,'施工单已使用排期，操作失败！'];
        try {
            DB::beginTransaction();

            $time = time();

            Schedule::where('id',$data['id'])->update(['del_time' => $time]);

            ScheduleDetail::where('schedule_id',$data['id'])
                ->where('del_time',0)
                ->update(['del_time' => $time]);

            ScheduleInfo::where('schedule_id',$data['id'])
                ->where('del_time',0)
                ->update(['del_time' => $time]);

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function detail($data,$user){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];
        $schedule = Schedule::where('del_time',0)
            ->where('id',$data['id'])
            ->first();
        if(empty($schedule)) return [false,'排班设置不存在或已被删除'];
        $schedule = $schedule->toArray();
        $schedule['choose'] = 1;

        $info = ScheduleDetail::where('del_time',0)
            ->where('schedule_id',$data['id'])
            ->select('start_time','end_time','num')
            ->get()->toArray();

        $schedule['detail'] = array_values($info);

        $schedule['crt_name'] = Employee::where('id',$schedule['crt_id'])->value('emp_name');
        $schedule['crt_time'] = $schedule['crt_time'] ? date("Y-m-d H:i:s",$schedule['crt_time']): '';

        return [true, $schedule];
    }

    //暂时不用
    public function detail2($data,$user){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];
        $schedule = Schedule::where('del_time',0)
            ->where('id',$data['id'])
            ->first();
        if(empty($schedule)) return [false,'排班设置不存在或已被删除'];
        $schedule = $schedule->toArray();

        $date1 = date("Y-m-d", $schedule['start_time']);
        $date2 = date("Y-m-d", $schedule['end_time']);
        $datetime1 = new \DateTime($date1);
        $datetime2 = new \DateTime($date2);
        $interval = $datetime1->diff($datetime2);
        $day = $interval->days + 1;

        $info = ScheduleInfo::where('del_time',0)
            ->where('schedule_id',$data['id'])
            ->select('start_time','end_time','num')
            ->get()->toArray();
        $infos = [];
        foreach ($info as $value){
            $start_time = date("H:i",$value['start_time']);
            $end_time = date("H:i",$value['end_time']);
            if(isset($infos[$start_time . $end_time])){
                $infos[$start_time . $end_time]['num'] += 1;
            }else{
                $infos[$start_time . $end_time] = [
                    'start_time' => $start_time,
                    'end_time' => $end_time,
                    'num' => 1,
                    'choose' => 1,
                ];
            }
        }
        foreach ($infos as $key => $value){
            $infos[$key]['num'] = $value['num'] / $day;
        }
        ksort($infos);
        $schedule['detail'] = array_values($infos);

        $schedule['crt_name'] = Employee::where('id',$schedule['crt_id'])->value('emp_name');
        $schedule['crt_time'] = $schedule['crt_time'] ? date("Y-m-d H:i:s",$schedule['crt_time']): '';

        return [true, $schedule];
    }

    public function getList($data,$user){
        $model = Schedule::TopClear($user,$data);
        $model = $model->where('del_time',0)
            ->select('id','crt_id','mark','start_time','end_time','crt_time')
            ->orderby('id', 'desc');

        if(! empty($data['mark'])) $model->where('mark', 'LIKE', '%'.$data['mark'].'%');
        if(! empty($data['crt_time'][0]) && ! empty($data['crt_time'][1])) {
            $return = $this->changeDateToTimeStampAboutRange($data['crt_time']);
            $model->where('crt_time','>=',$return[0]);
            $model->where('crt_time','<=',$return[1]);
        }
        if(! empty($data['schedule'][0]) && ! empty($data['schedule'][1])) {
            $return = $this->changeDateToTimeStampAboutRange($data['schedule']);
            $model->where('start_time','>=',$return[0]);
            $model->where('end_time','<=',$return[1]);
        }

        $list = $this->limit($model,'',$data);
        $list = $this->fillData($list,$user);

        return [true, $list];
    }

    public function scheduleRule(&$data, $user, $is_add = true){
        if(empty($data['schedule'][0]) || empty($data['schedule'][1])) return [false,'排班日期不能为空'];
        $data['start_time'] = $this->changeDateToDate($data['schedule'][0]);
        $data['end_time'] = $this->changeDateToDate($data['schedule'][1]);
        $data['time_range'] = $this->getTimestampRange($data['start_time'],$data['end_time']);
        if(empty($data['day_time'])) return [false,'排班时间段不能为空'];
        foreach ($data['day_time'] as $value){
            if(empty($value['start_time'])) return [false,'排班时间段开始时间不能为空'];
            if(empty($value['end_time'])) return [false,'排班时间段结束时间不能为空'];
            if(empty($value['num'])) return [false,'排班车间数量不能为空'];
        }

        if($is_add){
            $model = Schedule::TopClear($user,$data);
            $bool = $model->where('del_time',0)
                ->where('start_time', '<=', $data['end_time'])
                ->where('end_time', '>=', $data['start_time'])
                ->exists();
        }else{
            if(empty($data['id'])) return [false,'ID不能为空'];
            $model = Schedule::TopClear($user,$data);
            $bool = $model->where('del_time',0)
                ->where('id','<>',$data['id'])
                ->where('start_time', '<=', $data['end_time'])
                ->where('end_time', '>=', $data['start_time'])
                ->exists();

            $boolean = Construction::where('del_time',0)
                ->where('schedule_id',$data['id'])
                ->exists();
            if($boolean) return [false,'施工单已使用排期，编辑失败！'];
        }
        if($bool) return [false,'排班日期已经设置，请不要重复设置！'];

        //所属部门 以及 顶级部门
        if(empty($data['depart_id'])) $data['depart_id'] = $this->getDepart($user);
        $data['top_depart_id'] = $user['depart_map'][$data['depart_id']] ?? 0;

        return [true, $data];
    }

    public function fillData($data, $user){
        if(empty($data['data'])) return $data;

        $emp = Employee::whereIn('id',array_unique(array_column($data['data'],'crt_id')))
            ->pluck('emp_name','id')
            ->toArray();

        foreach ($data['data'] as $key => $value){
            $start_time = $value['start_time'] ? date("Y-m-d",$value['start_time']) : '';
            $end_time = $value['end_time'] ? date("Y-m-d",$value['end_time']) : '';
            $data['data'][$key]['schedule'] = $start_time . '——' . $end_time;
            $data['data'][$key]['crt_time'] = $value['crt_time'] ? date('Y-m-d H:i:s',$value['crt_time']) : '';
            $data['data'][$key]['crt_name'] = $emp[$value['crt_id']] ?? '';
        }

        return $data;
    }

    public function scheduleGetForConstruction($data,$user){
        if(empty($data['schedule'])) return [false,'请选择日期！'];
        $schedule = $this->changeDateToDate($data['schedule']);
        $model = Schedule::TopClear($user,$data);
        $schedule_id = $model->where('del_time',0)
            ->where('start_time', '<=', $schedule)
            ->where('end_time', '>=', $schedule)
            ->select('id')->get()->toArray();
        $schedule_id = array_column($schedule_id,'id');
        if(empty($schedule_id)) return [false,'该时间排班设置不存在或已被删除！'];

        $schedule_detail_list = ScheduleInfo::where('del_time',0)
            ->whereIn('schedule_id',$schedule_id)
            ->where('day',$schedule)
            ->where('is_use',ScheduleInfo::not_use)
            ->select('schedule_id','day','start_time','end_time')
            ->get()->toArray();
        if(empty($schedule_detail_list)) return [false,'该时间排班已满、不存在或已被删除！'];

        $return = [];
        foreach ($schedule_detail_list as $value){
            if(isset($return[$value['start_time'] . $value['end_time']])){
                $return[$value['start_time'] . $value['end_time']]['num'] += 1;
            }else{
                $tmp['schedule_id'] = $value['schedule_id'];
                $tmp['num'] = 1;
                $tmp['day_stamp'] = $value['day'];
                $tmp['day_start_stamp'] = $value['start_time'];
                $tmp['day_end_stamp'] = $value['end_time'];
                $tmp['day'] = date('Y-m-d',$value['day']);
                $tmp['start_time'] = date('H:i',$value['start_time']);
                $tmp['end_time'] = date('H:i',$value['end_time']);
                $return[$value['start_time'] . $value['end_time']] = $tmp;
            }
        }
        ksort($return);

        return [true, array_values($return)];
    }

    public function getTimestampRange($startTimestamp, $endTimestamp){
        // 将时间戳转换为日期格式
        $startDate = date('Y-m-d', $startTimestamp);
        $endDate = date('Y-m-d', $endTimestamp);

        // 创建日期范围
        $dateRange = [];
        $currentDate = strtotime($startDate);

        while ($currentDate <= strtotime($endDate)) {
            $dateRange[] = $currentDate;
            $currentDate = strtotime('+1 day', $currentDate);
        }

        // 将每个日期设置为当天的 0 点 0 分 0 秒并转换为时间戳
        $zeroTimestamps = [];

        foreach ($dateRange as $date) {
            $zeroTimestamps[] = strtotime(date('Y-m-d 00:00:00', $date));
        }

        return $zeroTimestamps;
    }

    public function getScheduleSetting($data,$user){
        $result['time'] = [];
        $result['time_detail'] = [];
        if(empty($data['schedule'])) return [true,$result];
        // 获取指定年月的开始日期时间戳
        $startTimestamp = strtotime($data['schedule'] . "-01");
        // 获取下个月的开始日期时间戳，然后减去一秒得到本月的结束日期时间戳
        $endTimestamp = strtotime("+1 month", $startTimestamp) - 1;

        $model = Schedule::TopClear($user,$data);
        $schedule = $model->where('del_time',0)
            ->where('start_time', '>=', $startTimestamp)
            ->where('end_time', '<=', $endTimestamp)
            ->select('id','mark','start_time','end_time')
            ->get()->toArray();
        if(empty($schedule)) return [true,$result];
        $schedule_array = $schedule_map = [];
        foreach ($schedule as $value){
            for ($i = $value['start_time']; $i <= $value['end_time']; $i += 86400) { // 每天86400秒
                $schedule_array[] = date("Y-m-d", $i);
                $schedule_map[$value['id']][] = date("Y-m-d", $i);
            }
        }

        $schedule_id = array_column($schedule,'id');
        $schedule_detail_info_list = ScheduleInfo::where('del_time',0)
            ->whereIn('schedule_id',$schedule_id)
            ->select('schedule_id','day','start_time','end_time')
            ->where('is_use',ScheduleInfo::not_use)
            ->get()->toArray();
        $schedule_detail_info_list_map = [];
        $count = [];
        if(! empty($schedule_detail_info_list)) {
            foreach ($schedule_detail_info_list as $value){
                $keys = $value['schedule_id'] . $value['start_time'] . $value['end_time'];
                if(isset($count[$keys])) continue;
                $count[$keys] = 1;
                $schedule_detail_info_list_map[$value['schedule_id']][] = [
                    "schedule_id" => $value['schedule_id'],
                    "day" => date('Y-m-d',$value['day']),
                    "start_time" => date("H:i",$value['start_time']),
                    "end_time" => date("H:i",$value['end_time']),
                    "day_stamp" => $value['day'],
                    "day_start_stamp" => $value['start_time'],
                    "day_end_stamp" => $value['end_time'],
                ];
            }
        }

        //每个设置时间的数据
        $return = [];
        foreach ($schedule_map as $key => $value){
            if(isset($schedule_detail_info_list_map[$key])){
                foreach ($value as $v){
                    $tmp = [];
                    foreach ($schedule_detail_info_list_map[$key] as $vv){
                        if($v == $vv['day']){
                            $tmp[] = [
                                "start_time" => $vv['start_time'],
                                "end_time" => $vv['end_time'],
                                "schedule_id" => $vv['schedule_id'],
                                "day_stamp" => $vv['day_stamp'],
                                "day_start_stamp" => $vv['day_start_stamp'],
                                "day_end_stamp" => $vv['day_end_stamp'],
                            ];
                        }
                    }
                    if(! empty($tmp)){
                        $return[$v] = [
                            'time' => $v,
                            'detail' => $tmp
                        ];
                    }
                }
            }
        }

        //已满日期
        $full = [];
        foreach ($schedule_array as $value){
            if(! isset($return[$value])) $full[] = $value;
        }

        $result['time'] = $full;
        $result['time_detail'] = array_values($return);

        return [true, $result];
    }
}
